@@ -1,5 +1,6 @@
 import boto3
 from cloud_governance.common.logger.init_logger import logger
+from cloud_governance.common.aws.utils.utils import Utils
 
 
 # @todo add next token
@@ -8,6 +9,9 @@ from cloud_governance.common.logger.init_logger import logger
 # while "NextToken" in response:
 #     response = client.get_servers(NextToken=response["NextToken"])
 #     results.extend(response["serverList"])
+from cloud_governance.zombie_cluster.delete_ec2_resources import DeleteEC2Resources
+from cloud_governance.zombie_cluster.delete_iam_resources import DeleteIAMResources
+from cloud_governance.zombie_cluster.delete_s3_resources import DeleteS3Resources
 
 
 class ZombieClusterResources:
@@ -15,7 +19,8 @@ class ZombieClusterResources:
     This class filter zombie cluster resources
     """
 
-    def __init__(self, cluster_prefix: str = None, delete: bool = False, region: str = 'us-east-2', cluster_tag: str = ''):
+    def __init__(self, cluster_prefix: str = None, delete: bool = False, region: str = 'us-east-2',
+                 cluster_tag: str = '', resource_name: str = ''):
         self.ec2_client = boto3.client('ec2', region_name=region)
         self.ec2_resource = boto3.resource('ec2', region_name=region)
         self.elb_client = boto3.client('elb', region_name=region)
@@ -26,6 +31,12 @@ class ZombieClusterResources:
         self.cluster_prefix = cluster_prefix
         self.delete = delete
         self.cluster_tag = cluster_tag
+        self.resource_name = resource_name
+        self.delete_ec2_resource = DeleteEC2Resources(self.ec2_client, self.elb_client, self.elbv2_client)
+        self.delete_iam_resource = DeleteIAMResources(iam_client=self.iam_client)
+        self.delete_s3_resource = DeleteS3Resources(s3_client=self.s3_client, s3_resource=self.s3_resource)
+        self.__get_details_resource_list = Utils().get_details_resource_list
+
 
     def _all_cluster_instance(self):
         """
@@ -93,13 +104,38 @@ class ZombieClusterResources:
                 for tag in resource[tags]:
                     if tag['Key'].startswith(self.cluster_prefix):
                         # when input a specific cluster, return resource id of the input cluster
-                        if self.cluster_tag:
-                            if self.cluster_tag == tag['Key']:
+                        for inner_tag in resource[tags]:
+                            if self.cluster_tag:
+                                if self.cluster_tag == inner_tag['Key']:
+                                    result_resources_key_id[resource_id] = tag['Key']
+                            elif self.resource_name:
+                                if self.resource_name == inner_tag['Value']:
+                                    result_resources_key_id[resource_id] = tag['Key']
+                            else:
                                 result_resources_key_id[resource_id] = tag['Key']
-                        else:
-                            result_resources_key_id[resource_id] = tag['Key']
+                        break
         return result_resources_key_id
 
+    def __extract_vpc_id_from_resource_data(self, zombie_id: str, resource_data: list, input_tag: str, output_tag: str =''):
+        vpc_id = ''
+        for resource in resource_data:
+            if resource.get(input_tag) == zombie_id:
+                if output_tag:
+                    vpc_id = resource.get(output_tag)[0].get('VpcId')
+                elif resource[input_tag] == zombie_id:
+                    vpc_id = resource.get('VpcId')
+        return vpc_id
+
+    def __get_cluster_resources_by_vpc_id(self, vpc_id: str, resource_data: list, output_tag: str, input_tag: str = ''):
+        ids = []
+        for resource in resource_data:
+            if input_tag:
+                if resource.get(input_tag)[0].get('VpcId') == vpc_id:
+                    ids.append(resource.get(output_tag))
+            elif resource.get('VpcId') == vpc_id:
+                ids.append(resource.get(output_tag))
+        return ids
+
     def __get_zombie_resources(self, exist_resources: dict):
         """
         This method filter zombie resource, meaning no active instance for this cluster
@@ -133,22 +169,18 @@ class ZombieClusterResources:
         delete only available resource that related to cluster
         """
         available_volumes = []
-        volumes = self.ec2_client.describe_volumes()
-        volumes_data = volumes['Volumes']
+        volumes_data = self.__get_details_resource_list(self.ec2_client.describe_volumes, input_tag='Volumes',
+                                                        check_tag='NextToken')
         # filter in-use resource
         for volume in volumes_data:
             if volume['State'] == 'available':
                 available_volumes.append(volume)
         exist_volume = self.__get_cluster_resources(resources_list=available_volumes, input_resource_id='VolumeId')
         zombies = self.__get_zombie_resources(exist_volume)
+
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_volume(VolumeId=zombie)
-                    logger.info(f'delete_volume: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_volume: {zombie}, {err}')
-
+                self.delete_ec2_resource.delete_zombie_resource(resource_id=zombie, resource='ec2_volume')
         return zombies
 
     def zombie_cluster_ami(self):
@@ -175,15 +207,11 @@ class ZombieClusterResources:
         snapshots = self.ec2_client.describe_snapshots(OwnerIds=['self'])
         snapshots_data = snapshots['Snapshots']
         exist_snapshot = self.__get_cluster_resources(resources_list=snapshots_data,
-                                                             input_resource_id='SnapshotId')
+                                                      input_resource_id='SnapshotId')
         zombies = self.__get_zombie_resources(exist_snapshot)
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_snapshot(SnapshotId=zombie)
-                    logger.info(f'delete_snapshot: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_snapshot: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='ebs_snapshots', resource_id=zombie)
         return zombies
 
     def zombie_cluster_security_group(self):
@@ -191,17 +219,20 @@ class ZombieClusterResources:
         This method return list of zombie cluster's security groups compare to existing instances and cluster name data
         :return: list of zombie cluster's security groups
         """
-        security_groups = self.ec2_client.describe_security_groups()
-        exist_security_group = self.__get_cluster_resources(resources_list=security_groups['SecurityGroups'],
-                                            input_resource_id='GroupId')
+        security_groups = self.__get_details_resource_list(func_name=self.ec2_client.describe_security_groups,
+                                                           input_tag='SecurityGroups', check_tag='NextToken')
+        exist_security_group = self.__get_cluster_resources(resources_list=security_groups,
+                                                            input_resource_id='GroupId')
         zombies = self.__get_zombie_resources(exist_security_group)
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_security_group(GroupId=zombie)
-                    logger.info(f'delete_security_group: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_security_group: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=security_groups,
+                                                              input_tag='GroupId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=security_groups,
+                                                                output_tag='GroupId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource('security_group', resource_id=zombie, vpc_id=vpc_id)
         return zombies
 
     def zombie_cluster_elastic_ip(self):
@@ -215,48 +246,43 @@ class ZombieClusterResources:
         for elastic_ip in elastic_ips_data:
             if elastic_ip.get('AssociationId'):
                 exist_elastic_ip_association.append(elastic_ip)
-            elif elastic_ip.get('AllocationId'):
+            if elastic_ip.get('AllocationId'):
                 exist_elastic_ip_allocation.append(elastic_ip)
         exist_elastic_ip_ass = self.__get_cluster_resources(resources_list=exist_elastic_ip_association,
-                                                             input_resource_id='AssociationId')
+                                                            input_resource_id='AssociationId')
         exist_elastic_ip_all = self.__get_cluster_resources(resources_list=exist_elastic_ip_allocation,
-                                                             input_resource_id='AllocationId')
+                                                            input_resource_id='AllocationId')
         zombies_ass = self.__get_zombie_resources(exist_elastic_ip_ass)
         zombies_all = self.__get_zombie_resources(exist_elastic_ip_all)
         if zombies_ass and self.delete:
             for zombie in zombies_ass:
-                try:
-                    self.ec2_client.disassociate_address(AssociationId=zombie)
-                    logger.info(f'disassociate_address: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot disassociate_address: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='elastic_ip', resource_id=zombie, deletion_type='disassociate')
         if zombies_all and self.delete:
             for zombie in zombies_all:
-                try:
-                    self.ec2_client.release_address(AllocationId=zombie)
-                    logger.info(f'release_address: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot release_address: {zombie}, {err}')
-        zombies = {**zombies_ass, **zombies_all}
+                self.delete_ec2_resource.delete_zombie_resource(resource='elastic_ip', resource_id=zombie)
+        zombies = {**zombies_all}
         return zombies
 
     def zombie_cluster_network_interface(self):
         """
         This method return list of zombie cluster's network interface according to existing instances and cluster name data
         """
-        network_interfaces = self.ec2_client.describe_network_interfaces()
-        network_interfaces_data = network_interfaces['NetworkInterfaces']
+        network_interfaces_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_network_interfaces,
+                                                                   input_tag='NetworkInterfaces', check_tag='NextToken')
         exist_network_interface = self.__get_cluster_resources(resources_list=network_interfaces_data,
-                                                             input_resource_id='NetworkInterfaceId',
-                                                             tags='TagSet')
+                                                               input_resource_id='NetworkInterfaceId',
+                                                               tags='TagSet')
         zombies = self.__get_zombie_resources(exist_network_interface)
+
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_network_interface(NetworkInterfaceId=zombie)
-                    logger.info(f'delete_network_interface: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot disassociate_address: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=network_interfaces_data,
+                                                              input_tag='NetworkInterfaceId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=network_interfaces_data,
+                                                                output_tag='NetworkInterfaceId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='network_interface', resource_id=zombie)
         return zombies
 
     def zombie_cluster_load_balancer(self):
@@ -284,11 +310,7 @@ class ZombieClusterResources:
         zombies = self.__get_zombie_resources(exist_load_balancer)
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_load_balancer(LoadBalancerName=zombie)
-                    logger.info(f'delete_load_balancer: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_load_balancer: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer', resource_id=zombie)
         return zombies
 
     def zombie_cluster_load_balancer_v2(self):
@@ -315,11 +337,7 @@ class ZombieClusterResources:
         zombies = self.__get_zombie_resources(exist_load_balancer)
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_load_balancer(LoadBalancerArn=zombie)
-                    logger.info(f'delete_load_balancer: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_load_balancer: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer_v2', resource_id=zombie)
         return zombies
 
     def __get_all_exist_vpcs(self):
@@ -327,81 +345,95 @@ class ZombieClusterResources:
         This method return all exist vpc ids (for supporting Network ACL - missing OpenShift tags)
         :return:
         """
-        vpcs = self.ec2_client.describe_vpcs()
-        vpcs_data = vpcs['Vpcs']
+        vpcs_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpcs, input_tag='Vpcs',
+                                                     check_tag='NextToken')
         vpcs_list = []
         for resource in vpcs_data:
             vpcs_list.append(resource['VpcId'])
         return vpcs_list
 
-    def zombie_cluster_vpc(self):
+    def zombie_cluster_vpc(self, ):
         """
         This method return list of cluster's vpc according to cluster tag name and cluster name data
         """
-        vpcs = self.ec2_client.describe_vpcs()
-        vpcs_data = vpcs['Vpcs']
+        vpcs_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpcs, input_tag='Vpcs',
+                                                     check_tag='NextToken')
         exist_vpc = self.__get_cluster_resources(resources_list=vpcs_data,
-                                                             input_resource_id='VpcId')
+                                                 input_resource_id='VpcId')
         zombies = self.__get_zombie_resources(exist_vpc)
+        delete_dict = {"ELB": self.zombie_cluster_load_balancer, "ELBV2": self.zombie_cluster_load_balancer_v2,
+                       "VPCE": self.zombie_cluster_vpc_endpoint, "DHCP": self.zombie_cluster_dhcp_option,
+                       "RT": self.zombie_cluster_route_table, "SG": self.zombie_cluster_security_group,
+                       "NATG": self.zombie_cluster_nat_gateway, "NACL": self.zombie_network_acl,
+                       "ENI": self.zombie_cluster_network_interface, "IGW": self.zombie_cluster_internet_gateway,
+                       "SUB": self.zombie_cluster_subnet, "EIP": self.zombie_cluster_elastic_ip}
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_vpc(VpcId=zombie)
-                    logger.info(f'delete_vpc: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_vpc: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='vpc', resource_id=zombie,
+                                                                pending_resources=delete_dict)
         return zombies
 
     def zombie_cluster_subnet(self):
         """
         This method return list of cluster's subnet according to cluster tag name and cluster name data
         """
-        subnets = self.ec2_client.describe_subnets()
-        subnets_data = subnets['Subnets']
+        subnets_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_subnets, input_tag='Subnets',
+                                                        check_tag='NextToken')
         exist_subnet = self.__get_cluster_resources(resources_list=subnets_data,
-                                                             input_resource_id='SubnetId')
+                                                    input_resource_id='SubnetId')
         zombies = self.__get_zombie_resources(exist_subnet)
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_subnet(SubnetId=zombie)
-                    logger.info(f'delete_subnet: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_subnet: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=subnets_data,
+                                                              input_tag='SubnetId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=subnets_data,
+                                                                output_tag='SubnetId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='subnet', resource_id=zombie)
         return zombies
 
     def zombie_cluster_route_table(self):
         """
         This method return list of cluster's route table according to cluster tag name and cluster name data
         """
-        route_tables = self.ec2_client.describe_route_tables()
-        route_tables_data = route_tables['RouteTables']
+        route_tables_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_route_tables,
+                                                             input_tag='RouteTables', check_tag='NextToken')
         exist_route_table = self.__get_cluster_resources(resources_list=route_tables_data,
-                                                             input_resource_id='RouteTableId')
+                                                         input_resource_id='RouteTableId')
         zombies = self.__get_zombie_resources(exist_route_table)
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_route_table(RouteTableId=zombie)
-                    logger.info(f'delete_route_table: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_route_table: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=route_tables_data,
+                                                              input_tag='RouteTableId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=route_tables_data,
+                                                                output_tag='RouteTableId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='route_table', resource_id=zombie, vpc_id=vpc_id)
         return zombies
 
     def zombie_cluster_internet_gateway(self):
         """
         This method return list of cluster's route table internet gateway according to cluster tag name and cluster name data
         """
-        internet_gateways = self.ec2_client.describe_internet_gateways()
-        internet_gateways_data = internet_gateways['InternetGateways']
+        internet_gateways_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_internet_gateways,
+                                                                  input_tag='InternetGateways', check_tag='NextToken')
         exist_internet_gateway = self.__get_cluster_resources(resources_list=internet_gateways_data,
-                                                             input_resource_id='InternetGatewayId')
+                                                              input_resource_id='InternetGatewayId')
+
         zombies = self.__get_zombie_resources(exist_internet_gateway)
         if zombies and self.delete:
-            for zombie in zombies:
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=internet_gateways_data,
+                                                              input_tag='InternetGatewayId', output_tag='Attachments')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=internet_gateways_data,
+                                                                output_tag='InternetGatewayId', input_tag='Attachments')
+            for zombie in zombie_ids:
                 try:
-                    self.ec2_client.delete_internet_gateway(InternetGatewayId=zombie)
-                    logger.info(f'delete_internet_gateway: {zombie}')
+                    self.delete_ec2_resource.delete_zombie_resource(resource='internet_gateway', resource_id=zombie,
+                                                                    vpc_id=vpc_id)
                 except Exception as err:
                     logger.exception(f'Cannot delete_internet_gateway: {zombie}, {err}')
         return zombies
@@ -410,63 +442,64 @@ class ZombieClusterResources:
         """
         This method return list of cluster's dhcp option according to cluster tag name and cluster name data
         """
-        dhcp_options = self.ec2_client.describe_dhcp_options()
-        dhcp_options_data = dhcp_options['DhcpOptions']
+        dhcp_options_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_dhcp_options,
+                                                             input_tag='DhcpOptions', check_tag='NextToken')
         exist_dhcp_option = self.__get_cluster_resources(resources_list=dhcp_options_data,
-                                                             input_resource_id='DhcpOptionsId')
+                                                         input_resource_id='DhcpOptionsId')
         zombies = self.__get_zombie_resources(exist_dhcp_option)
         if zombies and self.delete:
+            vpcs = self.ec2_client.describe_vpcs()['Vpcs']
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0], resource_data=vpcs,
+                                                              input_tag='DhcpOptionsId')
             for zombie in zombies:
-                try:
-                    self.ec2_client.delete_dhcp_options(DhcpOptionsId=zombie)
-                    logger.info(f'delete_internet_gateway: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_internet_gateway: {zombie}, {err}')
+                self.delete_ec2_resource.delete_zombie_resource(resource='dhcp_options', resource_id=zombie, vpc_id=vpc_id)
         return zombies
 
     def zombie_cluster_vpc_endpoint(self):
         """
         This method return list of cluster's vpc endpoint according to cluster tag name and cluster name data
         """
-        vpc_endpoints = self.ec2_client.describe_vpc_endpoints()
-        vpc_endpoints_data = vpc_endpoints['VpcEndpoints']
+        vpc_endpoints_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpc_endpoints,
+                                                              input_tag='VpcEndpoints', check_tag='NextToken')
         exist_vpc_endpoint = self.__get_cluster_resources(resources_list=vpc_endpoints_data,
-                                                             input_resource_id='VpcEndpointId')
+                                                          input_resource_id='VpcEndpointId')
         zombies = self.__get_zombie_resources(exist_vpc_endpoint)
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_vpc_endpoints(VpcEndpointIds=[zombie])
-                    logger.info(f'delete_vpc_endpoints: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_vpc_endpoints: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                              resource_data=vpc_endpoints_data,
+                                                              input_tag='VpcEndpointId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
+                                                                resource_data=vpc_endpoints_data,
+                                                                output_tag='VpcEndpointId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='vpc_endpoints', resource_id=zombie)
         return zombies
 
     def zombie_cluster_nat_gateway(self):
         """
         This method return list of zombie cluster's nat gateway according to cluster tag name and cluster name data
         """
-        nat_gateways = self.ec2_client.describe_nat_gateways()
-        nat_gateways_data = nat_gateways['NatGateways']
+        nat_gateways_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_nat_gateways,
+                                                             input_tag='NatGateways', check_tag='NextToken')
         exist_nat_gateway = self.__get_cluster_resources(resources_list=nat_gateways_data,
-                                                             input_resource_id='NatGatewayId')
+                                                         input_resource_id='NatGatewayId')
         zombies = self.__get_zombie_resources(exist_nat_gateway)
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_nat_gateway(NatGatewayId=zombie)
-                    logger.info(f'delete_nat_gateway: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_nat_gateway: {zombie}, {err}')
+            vpc_id = self.__extract_vpc_id_from_resource_data(list(zombies.keys())[0], nat_gateways_data,
+                                                              input_tag='NatGatewayId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id, resource_data=nat_gateways_data,
+                                                                output_tag='NatGatewayId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='nat_gateways', resource_id=zombie)
         return zombies
 
-    def zombie_network_acl(self):
+    def zombie_network_acl(self, vpc_id: str = ''):
         """
         This method return list of zombie cluster's network acl according to existing vpc id and cluster name data
         """
         exist_network_acl = {}
-        network_acls = self.ec2_client.describe_network_acls()
-        network_acls_data = network_acls['NetworkAcls']
+        network_acls_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_network_acls,
+                                                             input_tag='NetworkAcls', check_tag='NextToken')
         for resource in network_acls_data:
             exist_network_acl[resource['NetworkAclId']] = resource['VpcId']
         zombie_resources = {}
@@ -477,13 +510,18 @@ class ZombieClusterResources:
                 if zombie_value == value:
                     zombie_resources[key] = value
         zombies = zombie_resources
+        if vpc_id:
+            zombies = [network_acl.get('NetworkAclId') for network_acl in network_acls_data
+                       if network_acl.get('VpcId') == vpc_id]
         if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    self.ec2_client.delete_network_acl(NetworkAclId=zombie)
-                    logger.info(f'delete_network_acl: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_network_acl: {zombie}, {err}')
+            if not vpc_id:
+                vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=list(zombies.keys())[0],
+                                                                  resource_data=network_acls_data,
+                                                                  input_tag='NetworkAclId')
+            zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id, resource_data=network_acls_data,
+                                                                output_tag='NetworkAclId')
+            for zombie in zombie_ids:
+                self.delete_ec2_resource.delete_zombie_resource(resource='network_acl', resource_id=zombie, vpc_id=vpc_id)
         return zombies
 
     def zombie_cluster_role(self):
@@ -492,32 +530,29 @@ class ZombieClusterResources:
         * Role is a global resource, need to scan for live cluster in all regions
         """
         exist_role_name_tag = {}
-        roles = self.iam_client.list_roles()
-        roles_data = roles['Roles']
+        roles_data = self.__get_details_resource_list(func_name=self.iam_client.list_roles, input_tag='Roles',
+                                                      check_tag='Marker')
         for role in roles_data:
             role_name = role['RoleName']
-            role_data = self.iam_client.get_role(RoleName=role_name)
-            data = role_data['Role']
-            if data.get('Tags'):
-                for tag in data['Tags']:
-                    if tag['Key'].startswith(self.cluster_prefix):
-                        # when input a specific cluster, return resource id of the input cluster
-                        if self.cluster_tag:
-                            if self.cluster_tag == tag['Key']:
+            if 'worker-role' in role_name or 'master-role' in role_name:
+                role_data = self.iam_client.get_role(RoleName=role_name)
+                data = role_data['Role']
+                if data.get('Tags'):
+                    for tag in data['Tags']:
+                        if tag['Key'].startswith(self.cluster_prefix):
+                            # when input a specific cluster, return resource id of the input cluster
+                            if self.cluster_tag:
+                                if self.cluster_tag == tag['Key']:
+                                    exist_role_name_tag[role_name] = tag['Key']
+                            else:
                                 exist_role_name_tag[role_name] = tag['Key']
-                        else:
-                            exist_role_name_tag[role_name] = tag['Key']
-        zombies = self.__get_all_zombie_resources(exist_role_name_tag)
-        if zombies and self.delete:
-            for zombie in zombies:
-                try:
-                    # Detach policy from role
-                    self.iam_client.delete_role_policy(RoleName=zombie, PolicyName=zombie.replace('role', 'policy'))
-                    self.iam_client.remove_role_from_instance_profile(RoleName=zombie, InstanceProfileName=zombie.replace('role', 'profile'))
-                    self.iam_client.delete_role(RoleName=zombie)
-                    logger.info(f'delete_role: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_role: {zombie}, {err}')
+                            break
+        zombies = []
+        if exist_role_name_tag:
+            zombies = self.__get_all_zombie_resources(exist_role_name_tag)
+            if zombies and self.delete:
+                for zombie in zombies:
+                    self.delete_iam_resource.delete_iam_zombie_resource(resource_id=zombie, resource_type='iam_role')
         return zombies
 
     def zombie_cluster_user(self):
@@ -526,8 +561,8 @@ class ZombieClusterResources:
         * User is a global resource, need to scan for live cluster in all regions
         """
         exist_user_name_tag = {}
-        users = self.iam_client.list_users()
-        users_data = users['Users']
+        users_data = self.__get_details_resource_list(func_name=self.iam_client.list_users, input_tag='Users',
+                                                      check_tag='Marker')
         for user in users_data:
             user_name = user['UserName']
             user_data = self.iam_client.get_user(UserName=user_name)
@@ -541,22 +576,11 @@ class ZombieClusterResources:
                                 exist_user_name_tag[user_name] = tag['Key']
                         else:
                             exist_user_name_tag[user_name] = tag['Key']
+                        break
         zombies = self.__get_all_zombie_resources(exist_user_name_tag)
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    # Detach policy from user
-                    user_policies = self.iam_client.list_user_policies(UserName=zombie)
-                    if user_policies['PolicyNames']:
-                        self.iam_client.delete_user_policy(UserName=zombie, PolicyName=f'{zombie}-policy')
-                    list_access_key = self.iam_client.list_access_keys(UserName=zombie)
-                    # delete user access key
-                    for access_key in list_access_key['AccessKeyMetadata']:
-                        self.iam_client.delete_access_key(UserName=zombie, AccessKeyId=access_key['AccessKeyId'])
-                    self.iam_client.delete_user(UserName=zombie)
-                    logger.info(f'delete_user: {zombie}')
-                except Exception as err:
-                     logger.exception(f'Cannot delete_user: {zombie}, {err}')
+                self.delete_iam_resource.delete_iam_zombie_resource(resource_id=zombie, resource_type='iam_user')
         return zombies
 
     def zombie_cluster_s3_bucket(self, cluster_stamp: str = 'image-registry'):
@@ -573,7 +597,7 @@ class ZombieClusterResources:
             if cluster_stamp in bucket['Name']:
                 try:
                     tags = self.s3_client.get_bucket_tagging(Bucket=bucket['Name'])
-                except Exception:  # continue when no bucket tags
+                except Exception as e:  # continue when no bucket tags
                     continue
                 for tag in tags['TagSet']:
                     if tag['Key'].startswith(self.cluster_prefix):
@@ -583,19 +607,12 @@ class ZombieClusterResources:
                                 exist_bucket_name_tag[bucket['Name']] = tag['Key']
                         else:
                             exist_bucket_name_tag[bucket['Name']] = tag['Key']
+                        break
         zombies = self.__get_all_zombie_resources(exist_bucket_name_tag)
         if zombies and self.delete:
             for zombie in zombies:
-                try:
-                    # delete bucket objects
-                    bucket = self.s3_resource.Bucket(zombie)
-                    bucket.objects.all().delete()
-                    # delete bucket
-                    self.s3_client.delete_bucket(Bucket=zombie)
-                    logger.info(f'delete_bucket: {zombie}')
-                except Exception as err:
-                    logger.exception(f'Cannot delete_bucket: {zombie}, {err}')
+                self.delete_s3_resource.delete_zombie_s3_resource(resource_type='s3_bucket', resource_id=zombie)
         return zombies
 
 # zombie_cluster_resources = ZombieClusterResources(cluster_prefix='kubernetes.io/cluster/', delete=False, region='us-east-2')
-# print(zombie_cluster_resources.zombie_cluster_subnet())
\ No newline at end of file
+# print(zombie_cluster_resources.zombie_cluster_subnet())
